@@ -7,7 +7,7 @@ import matplotsoccer as mpl
 
 
 class PitchControl:
-    def __init__(self, tracking: pd.DataFrame, events: pd.DataFrame):
+    def __init__(self, tracking: pd.DataFrame, events: pd.DataFrame, ball_data: pd.DataFrame = None):
         self.team_ids = tracking['team_id'].unique()
         sides = tracking.groupby('team_id')['side_1h'].unique()
         side_by_team = dict(zip(self.team_ids, sides[self.team_ids].apply(lambda x: x[0])))
@@ -22,10 +22,11 @@ class PitchControl:
 
         self.locs_home, self.locs_away, self.locs_ball, self.t = self.get_locs(
             tracking,
-            events
+            events,
+            ball_data
         )
 
-    def get_locs(self, tracking: pd.DataFrame, events: pd.DataFrame) -> tuple:
+    def get_locs(self, tracking: pd.DataFrame, events: pd.DataFrame, ball_data: pd.DataFrame | None) -> tuple:
         events = events[[
             'possession_number', 'team_id', 'possession_team_id',
             'half', 'second', 'pos_x', 'pos_y'
@@ -38,10 +39,11 @@ class PitchControl:
             lambda x: self.swap_coords(x, 'y'), axis=1
         )
 
-        ball_data = self.interpolate_ball_data(
-            events[['half', 'second', 'pos_x', 'pos_y']],
-            tracking
-        )
+        if ball_data is None:
+            ball_data = self.interpolate_ball_data(
+                events[['half', 'second', 'pos_x', 'pos_y']],
+                tracking
+            )
 
         locs_home = {
             half: {
@@ -124,10 +126,21 @@ class PitchControl:
 
     @staticmethod
     def get_player_data(player_id, half, tracking):
-        return tracking[
+        timestamps = tracking[tracking['half'] == half]['second'].unique()
+        player_data = tracking[
             (tracking['player_id'] == player_id)
             & (tracking['half'] == half)
-            ][['pos_x', 'pos_y']].values
+        ][['second', 'pos_x', 'pos_y']]
+
+        player_data_full = pd.DataFrame({'second': timestamps})
+        player_data_full = player_data_full.merge(player_data, on='second', how='left')
+
+        return player_data_full[['pos_x', 'pos_y']].values
+
+        # return tracking[
+        #     (tracking['player_id'] == player_id)
+        #     & (tracking['half'] == half)
+        #     ][['pos_x', 'pos_y']].values
 
     def influence_function(
         self,
@@ -195,11 +208,11 @@ class PitchControl:
         locations = np.c_[xx.flatten(),yy.flatten()]
 
         for k in self.locs_home[half].keys():
-            if len(self.locs_home[half][k]) >= tp:
-                Zh += self.influence_function(k, locations, tp, 'h', half)
+            # if len(self.locs_home[half][k]) >= tp:
+            Zh += self.influence_function(k, locations, tp, 'h', half)
         for k in self.locs_away[half].keys():
-            if len(self.locs_away[half][k]) >= tp:
-                Za += self.influence_function(k, locations, tp, 'a', half)
+            # if len(self.locs_away[half][k]) >= tp:
+            Za += self.influence_function(k, locations, tp, 'a', half)
 
         Zh = Zh.reshape((dt, dt))
         Za = Za.reshape((dt, dt))
@@ -225,7 +238,8 @@ class PitchControl:
         plt.contourf(xx, yy, pitch_control)
 
         for k in self.locs_home[half].keys():
-            if len(self.locs_home[half][k]) >= tp:
+            # if len(self.locs_home[half][k]) >= tp:
+            if np.isfinite(self.locs_home[half][k][tp, :]).all():
                 plt.scatter(
                     self.locs_home[half][k][tp, 0],
                     self.locs_home[half][k][tp, 1],
@@ -233,7 +247,8 @@ class PitchControl:
                 )
 
         for k in self.locs_away[half].keys():
-            if len(self.locs_away[half][k]) >= tp:
+            # if len(self.locs_away[half][k]) >= tp:
+            if np.isfinite(self.locs_away[half][k][tp, :]).all():
                 plt.scatter(
                     self.locs_away[half][k][tp, 0],
                     self.locs_away[half][k][tp, 1], color='black'
@@ -266,7 +281,7 @@ class PitchControl:
 
         def animate(i):
             fr = tp + i
-            pitch_control, xx, yy = self.fit(half, tp, dt)
+            pitch_control, xx, yy = self.fit(half, fr, dt)
 
             mpl.field("white", show=False, ax=ax)
             ax.axis('off')
@@ -274,14 +289,16 @@ class PitchControl:
             plt.contourf(xx, yy, pitch_control)
 
             for k in self.locs_home[half].keys():
-                if len(self.locs_home[half][k]) >= fr:
+                # if len(self.locs_home[half][k]) >= fr:
+                if np.isfinite(self.locs_home[half][k][fr, :]).all():
                     plt.scatter(
                         self.locs_home[half][k][fr, 0],
                         self.locs_home[half][k][fr, 1],
                         color='darkgrey'
                     )
             for k in self.locs_away[half].keys():
-                if len(self.locs_away[half][k]) >= fr:
+                # if len(self.locs_away[half][k]) >= fr:
+                if np.isfinite(self.locs_away[half][k][fr, :]).all():
                     plt.scatter(
                         self.locs_away[half][k][fr, 0],
                         self.locs_away[half][k][fr, 1],
